@@ -42,7 +42,7 @@
  * plugins, i.e. load or unload them, and see some pieces of information.
  *
  * The only thing you need to do as an application writer if you wish
- * to use the view to configure your plugins is to instantiate it using
+ * to use the view to display your plugins is to instantiate it using
  * peas_gtk_plugin_manager_view_new() and pack it into another
  * widget or a window.
  *
@@ -63,6 +63,7 @@ struct _PeasGtkPluginManagerViewPrivate {
 /* Properties */
 enum {
   PROP_0,
+  PROP_ENGINE,
   PROP_SHOW_BUILTIN
 };
 
@@ -189,23 +190,6 @@ enabled_toggled_cb (GtkCellRendererToggle    *cell,
   gtk_tree_path_free (path);
 }
 
-static void
-row_activated_cb (GtkTreeView              *tree_view,
-                  GtkTreePath              *path,
-                  GtkTreeViewColumn        *column,
-                  PeasGtkPluginManagerView *view)
-{
-  GtkTreeIter iter;
-
-  g_return_if_fail (gtk_tree_model_get_iter (gtk_tree_view_get_model (tree_view),
-                                             &iter, path));
-
-  convert_iter_to_child_iter (view, &iter);
-
-  if (peas_gtk_plugin_manager_store_can_enable (view->priv->store, &iter))
-    peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
-}
-
 /* Callback used as the interactive search comparison function */
 static gboolean
 name_search_cb (GtkTreeModel             *model,
@@ -295,7 +279,7 @@ create_popup_menu (PeasGtkPluginManagerView *view)
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
                                   peas_plugin_info_is_loaded (info));
   g_signal_connect (item, "toggled", G_CALLBACK (enabled_menu_cb), view);
-  gtk_widget_set_sensitive (item, peas_plugin_info_is_available (info) &&
+  gtk_widget_set_sensitive (item, peas_plugin_info_is_available (info, NULL) &&
                                   !peas_plugin_info_is_builtin (info));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -372,19 +356,23 @@ menu_position_under_tree_view (GtkMenu     *menu,
   else
     {
       GtkAllocation allocation;
+
       gtk_widget_get_allocation (GTK_WIDGET (tree_view), &allocation);
+
       *x += allocation.x;
       *y += allocation.y;
 
       if (gtk_widget_get_direction (GTK_WIDGET (tree_view)) == GTK_TEXT_DIR_RTL)
         {
           GtkRequisition requisition;
+
 #if !GTK_CHECK_VERSION(2,90,7)
           gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
 #else
           gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition,
                                          NULL);
 #endif
+
           *x += allocation.width - requisition.width;
         }
     }
@@ -425,37 +413,29 @@ show_popup_menu (GtkTreeView              *tree_view,
     }
 }
 
-static gboolean
-button_press_event_cb (GtkWidget                *tree_view,
-                       GdkEventButton           *event,
-                       PeasGtkPluginManagerView *view)
+static void
+plugin_icon_data_func (GtkTreeViewColumn *column,
+                       GtkCellRenderer   *cell,
+                       GtkTreeModel      *model,
+                       GtkTreeIter       *iter)
 {
-  GtkWidgetClass *widget_class;
-  gboolean handled;
+  GdkPixbuf *icon_pixbuf;
+  gchar *icon_name;
 
-  if (event->type != GDK_BUTTON_PRESS || event->button != 3)
-    return FALSE;
+  gtk_tree_model_get (model, iter,
+    PEAS_GTK_PLUGIN_MANAGER_STORE_ICON_PIXBUF_COLUMN, &icon_pixbuf,
+    PEAS_GTK_PLUGIN_MANAGER_STORE_ICON_NAME_COLUMN, &icon_name,
+    -1);
 
-  widget_class = GTK_WIDGET_CLASS (peas_gtk_plugin_manager_view_parent_class);
+  if (icon_pixbuf == NULL)
+    g_object_set (cell, "icon-name", icon_name, NULL);
+  else
+    {
+      g_object_set (cell, "pixbuf", icon_pixbuf, NULL);
+      g_object_unref (icon_pixbuf);
+    }
 
-  /* The selection must by updated */
-  handled = widget_class->button_press_event (tree_view, event);
-
-  if (!handled)
-    return FALSE;
-
-  show_popup_menu (GTK_TREE_VIEW (tree_view), view, event);
-
-  return TRUE;
-}
-
-static gboolean
-popup_menu_cb (GtkTreeView              *tree_view,
-               PeasGtkPluginManagerView *view)
-{
-  show_popup_menu (tree_view, view, NULL);
-
-  return TRUE;
+  g_free (icon_name);
 }
 
 static void
@@ -468,13 +448,6 @@ peas_gtk_plugin_manager_view_init (PeasGtkPluginManagerView *view)
                                             PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
                                             PeasGtkPluginManagerViewPrivate);
 
-  view->priv->store = peas_gtk_plugin_manager_store_new ();
-  view->priv->engine = g_object_ref (peas_engine_get_default ());
-  g_signal_connect (view->priv->engine,
-                    "notify::plugin-list",
-                    G_CALLBACK (plugin_list_changed_cb),
-                    view);
-
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
 
@@ -507,10 +480,9 @@ peas_gtk_plugin_manager_view_init (PeasGtkPluginManagerView *view)
   cell = gtk_cell_renderer_pixbuf_new ();
   gtk_tree_view_column_pack_start (column, cell, FALSE);
   g_object_set (cell, "stock-size", GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
-  gtk_tree_view_column_set_attributes (column, cell,
-                                       //"sensitive", PEAS_GTK_PLUGIN_MANAGER_STORE_CAN_ENABLE_COLUMN,
-                                       "icon-name", PEAS_GTK_PLUGIN_MANAGER_STORE_ICON_COLUMN,
-                                       NULL);
+  gtk_tree_view_column_set_cell_data_func (column, cell,
+                                           (GtkTreeCellDataFunc) plugin_icon_data_func,
+                                           NULL, NULL);
 
   cell = gtk_cell_renderer_text_new ();
   gtk_tree_view_column_pack_start (column, cell, TRUE);
@@ -530,21 +502,99 @@ peas_gtk_plugin_manager_view_init (PeasGtkPluginManagerView *view)
                                        (GtkTreeViewSearchEqualFunc) name_search_cb,
                                        view,
                                        NULL);
+}
 
-  gtk_widget_show (GTK_WIDGET (view));
+static gboolean
+peas_gtk_plugin_manager_view_button_press_event (GtkWidget      *tree_view,
+                                                 GdkEventButton *event)
+{
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view);
+  GtkWidgetClass *widget_class;
+  gboolean handled;
 
-  g_signal_connect (view,
-                    "row-activated",
-                    G_CALLBACK (row_activated_cb),
-                    view);
-  g_signal_connect (view,
-                    "button-press-event",
-                    G_CALLBACK (button_press_event_cb),
-                    view);
-  g_signal_connect (view,
-                    "popup-menu",
-                    G_CALLBACK (popup_menu_cb),
-                    view);
+  widget_class = GTK_WIDGET_CLASS (peas_gtk_plugin_manager_view_parent_class);
+
+  /* The selection must by updated */
+  handled = widget_class->button_press_event (tree_view, event);
+
+  if (event->type != GDK_BUTTON_PRESS || event->button != 3 || !handled)
+    return handled;
+
+  show_popup_menu (GTK_TREE_VIEW (tree_view), view, event);
+
+  return TRUE;
+}
+
+static gboolean
+peas_gtk_plugin_manager_view_popup_menu (GtkWidget *widget)
+{
+  GtkTreeView *tree_view = GTK_TREE_VIEW (widget);
+
+  show_popup_menu (tree_view, PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view), NULL);
+
+  return TRUE;
+}
+
+static gboolean
+peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
+                                            gint        x,
+                                            gint        y,
+                                            gboolean    keyboard_mode,
+                                            GtkTooltip *tooltip)
+{
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (widget);
+  gboolean is_row;
+  GtkTreeIter iter;
+  PeasPluginInfo *info;
+  GError *error = NULL;
+
+  is_row = gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (widget),
+                                              &x, &y, keyboard_mode,
+                                              NULL, NULL, &iter);
+
+  if (!is_row)
+    return FALSE;
+    
+  convert_iter_to_child_iter (view, &iter);
+
+  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, &iter);
+
+  if (!peas_plugin_info_is_available (info, &error))
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("<b>The plugin '%s' could not be loaded</b>\n"
+                                   "An error occurred: %s"),
+                                 peas_plugin_info_get_name (info),
+                                 error->message);
+
+      gtk_tooltip_set_markup (tooltip, message);
+
+      g_free (message);
+      g_error_free (error);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+  
+
+static void
+peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
+                                            GtkTreePath       *path,
+                                            GtkTreeViewColumn *column)
+{
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (tree_view);
+  GtkTreeIter iter;
+
+  if (!gtk_tree_model_get_iter (gtk_tree_view_get_model (tree_view), &iter, path))
+    return;
+
+  convert_iter_to_child_iter (view, &iter);
+
+  if (peas_gtk_plugin_manager_store_can_enable (view->priv->store, &iter))
+    peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
 }
 
 static void
@@ -557,9 +607,12 @@ peas_gtk_plugin_manager_view_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_ENGINE:
+      view->priv->engine = g_value_get_object (value);
+      break;
     case PROP_SHOW_BUILTIN:
       peas_gtk_plugin_manager_view_set_show_builtin (view,
-                                                    g_value_get_boolean (value));
+                                                     g_value_get_boolean (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -577,6 +630,9 @@ peas_gtk_plugin_manager_view_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_ENGINE:
+      g_value_set_object (value, view->priv->engine);
+      break;
     case PROP_SHOW_BUILTIN:
       g_value_set_boolean (value,
                            peas_gtk_plugin_manager_view_get_show_builtin (view));
@@ -592,10 +648,22 @@ peas_gtk_plugin_manager_view_constructed (GObject *object)
 {
   PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (object);
 
+  if (view->priv->engine == NULL)
+    view->priv->engine = peas_engine_get_default ();
+
+  g_object_ref (view->priv->engine);
+
+  view->priv->store = peas_gtk_plugin_manager_store_new (view->priv->engine);
+
   /* Properly set the model */
   view->priv->show_builtin = TRUE;
   peas_gtk_plugin_manager_view_set_show_builtin (view, FALSE);
 
+  g_signal_connect (view->priv->engine,
+                    "notify::plugin-list",
+                    G_CALLBACK (plugin_list_changed_cb),
+                    view);
+
   if (G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed != NULL)
     G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed (object);
 }
@@ -635,14 +703,37 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
 {
   GType the_type = G_TYPE_FROM_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
+  GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
+  GtkTreeViewClass *tree_view_class = GTK_TREE_VIEW_CLASS (klass);
 
   object_class->set_property = peas_gtk_plugin_manager_view_set_property;
   object_class->get_property = peas_gtk_plugin_manager_view_get_property;
   object_class->constructed = peas_gtk_plugin_manager_view_constructed;
   object_class->dispose = peas_gtk_plugin_manager_view_dispose;
 
+  widget_class->button_press_event = peas_gtk_plugin_manager_view_button_press_event;
+  widget_class->popup_menu = peas_gtk_plugin_manager_view_popup_menu;
+  widget_class->query_tooltip = peas_gtk_plugin_manager_view_query_tooltip;
+
+  tree_view_class->row_activated = peas_gtk_plugin_manager_view_row_activated;
+
   /**
-   * PeasGtkPLuginManagerView:show-builtin:
+   * PeasGtkPLuginManagerView:engine:
+   *
+   * The #PeasEngine this view is attached to.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_ENGINE,
+                                   g_param_spec_object ("engine",
+                                                        "engine",
+                                                        "The PeasEngine this view is attached to",
+                                                        PEAS_TYPE_ENGINE,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_STRINGS));
+
+  /**
+   * PeasGtkPluginManagerView:show-builtin:
    *
    * If builtin plugins should be shown.
    */
@@ -680,15 +771,21 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
 
 /**
  * peas_gtk_plugin_manager_view_new:
+ * @engine: (allow-none): A #PeasEngine, or %NULL.
  *
  * Creates a new plugin manager view for the given #PeasEngine.
  *
+ * If @engine is %NULL, then the default engine will be used.
+ *
  * Returns: the new #PeasGtkPluginManagerView.
  */
 GtkWidget *
-peas_gtk_plugin_manager_view_new (void)
+peas_gtk_plugin_manager_view_new (PeasEngine *engine)
 {
+  g_return_val_if_fail (engine == NULL || PEAS_IS_ENGINE (engine), NULL);
+
   return GTK_WIDGET (g_object_new (PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
+                                   "engine", engine,
                                    NULL));
 }
 
@@ -701,7 +798,7 @@ peas_gtk_plugin_manager_view_new (void)
  */
 void
 peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
-                                               gboolean                 show_builtin)
+                                               gboolean                  show_builtin)
 {
   GtkTreeIter iter;
   gboolean iter_set;
@@ -719,7 +816,7 @@ peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
 
   view->priv->show_builtin = show_builtin;
 
-  if (show_builtin == TRUE)
+  if (show_builtin)
     {
       gtk_tree_view_set_model (GTK_TREE_VIEW (view),
                                GTK_TREE_MODEL (view->priv->store));
@@ -741,6 +838,8 @@ peas_gtk_plugin_manager_view_set_show_builtin (PeasGtkPluginManagerView *view,
 
   if (iter_set)
     peas_gtk_plugin_manager_view_set_selected_iter (view, &iter);
+
+  g_object_notify (G_OBJECT (view), "show-builtin");
 }
 
 /**
@@ -768,7 +867,7 @@ peas_gtk_plugin_manager_view_get_show_builtin (PeasGtkPluginManagerView *view)
  */
 void
 peas_gtk_plugin_manager_view_set_selected_iter (PeasGtkPluginManagerView *view,
-                                                GtkTreeIter             *iter)
+                                                GtkTreeIter              *iter)
 {
   GtkTreeSelection *selection;
 
@@ -793,7 +892,7 @@ peas_gtk_plugin_manager_view_set_selected_iter (PeasGtkPluginManagerView *view,
  */
 gboolean
 peas_gtk_plugin_manager_view_get_selected_iter (PeasGtkPluginManagerView *view,
-                                                GtkTreeIter             *iter)
+                                                GtkTreeIter              *iter)
 {
   GtkTreeSelection *selection;
 
@@ -819,7 +918,7 @@ peas_gtk_plugin_manager_view_get_selected_iter (PeasGtkPluginManagerView *view,
  */
 void
 peas_gtk_plugin_manager_view_set_selected_plugin (PeasGtkPluginManagerView *view,
-                                                  PeasPluginInfo          *info)
+                                                  PeasPluginInfo           *info)
 {
   GtkTreeIter iter;
   GtkTreeSelection *selection;
@@ -828,7 +927,7 @@ peas_gtk_plugin_manager_view_set_selected_plugin (PeasGtkPluginManagerView *view
   g_return_if_fail (info != NULL);
 
   g_return_if_fail (peas_gtk_plugin_manager_store_get_iter_from_plugin (view->priv->store,
-                                                                       &iter, info));
+                                                                        &iter, info));
 
   if (!convert_child_iter_to_iter (view, &iter))
     return;
