@@ -73,8 +73,6 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   if (!g_atomic_int_dec_and_test (&info->refcount))
     return;
 
-  if (info->keys != NULL)
-    g_hash_table_destroy (info->keys);
   g_free (info->file);
   g_free (info->module_dir);
   g_free (info->data_dir);
@@ -89,6 +87,8 @@ _peas_plugin_info_unref (PeasPluginInfo *info)
   g_free (info->version);
   g_free (info->help_uri);
   g_strfreev (info->authors);
+  if (info->error != NULL)
+    g_error_free (info->error);
 
   g_free (info);
 }
@@ -106,76 +106,16 @@ peas_plugin_info_get_type (void)
   return the_type;
 }
 
-static void
-value_free (GValue *value)
+GQuark
+peas_plugin_info_error_quark (void)
 {
-  g_value_unset (value);
-  g_free (value);
-}
+  static volatile gsize quark = 0;
 
-static void
-parse_extra_keys (PeasPluginInfo   *info,
-                  GKeyFile         *plugin_file,
-                  const gchar     **keys)
-{
-  guint i;
+	if (g_once_init_enter (&quark))
+		g_once_init_leave (&quark,
+		                   g_quark_from_static_string ("peas-plugin-info-error"));
 
-  for (i = 0; keys[i] != NULL; i++)
-    {
-      GValue *value = NULL;
-      gboolean b;
-      GError *error = NULL;
-
-      if (g_str_equal (keys[i], "IAge") ||
-          g_str_equal (keys[i], "Module") ||
-          g_str_equal (keys[i], "Depends") ||
-          g_str_equal (keys[i], "Loader") ||
-          g_str_equal (keys[i], "Name") ||
-          g_str_has_prefix (keys[i], "Name[") ||
-          g_str_equal (keys[i], "Description") ||
-          g_str_has_prefix (keys[i], "Description[") ||
-          g_str_equal (keys[i], "Icon") ||
-          g_str_equal (keys[i], "Authors") ||
-          g_str_equal (keys[i], "Copyright") ||
-          g_str_equal (keys[i], "Website") ||
-          g_str_equal (keys[i], "Version") ||
-          g_str_has_prefix (keys[i], "Help") ||
-          g_str_equal (keys[i], "Builtin"))
-        continue;
-
-      b = g_key_file_get_boolean (plugin_file, "Plugin", keys[i], &error);
-      if (b == FALSE && error != NULL)
-        {
-          gchar *str;
-          g_error_free (error);
-          error = NULL;
-          str = g_key_file_get_string (plugin_file, "Plugin", keys[i], NULL);
-          if (str != NULL)
-            {
-              value = g_new0 (GValue, 1);
-              g_value_init (value, G_TYPE_STRING);
-              g_value_take_string (value, str);
-            }
-        }
-      else
-        {
-          value = g_new0 (GValue, 1);
-          g_value_init (value, G_TYPE_BOOLEAN);
-          g_value_set_boolean (value, b);
-        }
-
-      if (!value)
-        continue;
-
-      if (info->keys == NULL)
-        {
-          info->keys = g_hash_table_new_full (g_str_hash,
-                                              g_str_equal,
-                                              g_free,
-                                              (GDestroyNotify) value_free);
-        }
-      g_hash_table_insert (info->keys, g_strdup (keys[i]), value);
-    }
+	return quark;
 }
 
 /*
@@ -196,7 +136,6 @@ _peas_plugin_info_new (const gchar *filename,
   PeasPluginInfo *info;
   GKeyFile *plugin_file = NULL;
   gchar *str;
-  gchar **keys;
   gint integer;
   gboolean b;
   GError *error = NULL;
@@ -316,11 +255,6 @@ _peas_plugin_info_new (const gchar *filename,
   else
     info->builtin = b;
 
-  /* Get extra keys */
-  keys = g_key_file_get_keys (plugin_file, "Plugin", NULL, NULL);
-  parse_extra_keys (info, plugin_file, (const gchar **) keys);
-  g_strfreev (keys);
-
   g_key_file_free (plugin_file);
 
   info->module_dir = g_strdup (module_dir);
@@ -361,19 +295,28 @@ peas_plugin_info_is_loaded (const PeasPluginInfo *info)
 /**
  * peas_plugin_info_is_available:
  * @info: A #PeasPluginInfo.
+ * @error: A #GError.
  *
  * Check if the plugin is available.
  *
  * A plugin is marked as not available when there is no loader available to
  * load it, or when there has been an error when trying to load it previously.
+ * If not available then @error will be set.
  *
  * Returns: %TRUE if the plugin is available.
  */
 gboolean
-peas_plugin_info_is_available (const PeasPluginInfo *info)
+peas_plugin_info_is_available (const PeasPluginInfo  *info,
+                               GError               **error)
 {
   g_return_val_if_fail (info != NULL, FALSE);
 
+  /* Uses g_propagate_error() so we get the right warning
+   * in the case that *error != NULL
+   */
+  if (error != NULL && info->error != NULL)
+    g_propagate_error (error, g_error_copy (info->error));
+
   return info->available != FALSE;
 }
 
@@ -585,11 +528,11 @@ peas_plugin_info_get_icon_name (const PeasPluginInfo *info)
  * peas_plugin_info_get_authors:
  * @info: A #PeasPluginInfo.
  *
- * Gets a NULL-terminated array of strings with the authors of the plugin.
+ * Gets a %NULL-terminated array of strings with the authors of the plugin.
  *
  * The relevant key in the plugin info file is "Authors".
  *
- * Returns: (transfer none): the plugin's author list.
+ * Returns: (transfer none) (array zero-terminated=1): the plugin's author list.
  */
 const gchar **
 peas_plugin_info_get_authors (const PeasPluginInfo *info)
@@ -694,24 +637,3 @@ peas_plugin_info_get_iage (const PeasPluginInfo *info)
 
   return info->iage;
 }
-
-/**
- * peas_plugin_info_get_keys:
- * @info: A #PeasPluginInfo.
- *
- * Gets a hash table of string keys present and #GValue values,
- * present in the plugin information file, but not handled
- * by libpeas. Note that libpeas only handles booleans and
- * strings, and that strings that are recognized as booleans,
- * as done by g_key_file_get_boolean(), will be of boolean type.
- *
- * Returns: a #GHashTable of string keys and #GValue values. Do
- * not free or destroy any data in this hashtable.
- **/
-const GHashTable *
-peas_plugin_info_get_keys (const PeasPluginInfo *info)
-{
-  g_return_val_if_fail (info != NULL, NULL);
-
-  return info->keys;
-}
